'use strict';

const {Contract} = require('fabric-contract-api');

class Cnsp extends Contract {

    /**
     * 创建/更新数据
     */
    async upsert(ctx, datasAsBase64) {
        const datas = JSON.parse(Buffer.from(datasAsBase64, 'base64').toString());
        if (!Array.isArray(datas) || datas.length < 1) {
            throw new Error('数据格式错误，参数非数组格式或无数据');
        }
        datas.forEach(function (data) {
            if (!data.hasOwnProperty('docType')) {
                throw new Error('数据格式错误，参数无docType');
            }
        });
        for (let i = 0; i < datas.length; i++) {
            let data = datas[i];
            let key = data.key;
            delete data.key;
            await ctx.stub.putState(key, Buffer.from(JSON.stringify(data)));
        }
    }

    /**
     * 获取数据详情
     */
    async queryByKey(ctx, key) {
        console.info('获取数据详情：\n' + key);
        const dataAsBytes = await ctx.stub.getState(key);
        if (!dataAsBytes || dataAsBytes.length === 0) {
            throw new Error(`未找到KEY值为${key}的数据`);
        }

        return JSON.parse(dataAsBytes.toString());
    }

    /**
     * 删除数据
     */
    async deleteByKey(ctx, key) {
        console.info('删除数据：\n' + key);
        await ctx.stub.deleteState(key);
    }

    /**
     * 富查询
     */
    async queryByString(ctx, queryStringAsBase64) {
        const queryString = Buffer.from(queryStringAsBase64, 'base64').toString();
        if (!queryString) {
            throw new Error('查询字符串为空');
        }
        console.info('- queryString:\n' + queryString);
        let iterator = await ctx.stub.getQueryResult(queryString);
        return this.generateResults(iterator, false);
    }

    /**
     * 分页富查询
     */
    async queryByStringWithPagination(ctx, queryStringAsBase64, pageSize, bookmark) {
        const queryString = Buffer.from(queryStringAsBase64, 'base64').toString();
        if (!queryString) {
            throw new Error('查询字符串为空');
        }
        console.info('- queryString:\n' + queryString);
        let {
            iterator,
            metadata
        } = await ctx.stub.getQueryResultWithPagination(queryString, parseInt(pageSize, 10), bookmark);
        let results = this.generateResults(iterator, false);
        results.ResponseMetadata = {
            RecordsCount: metadata.fetched_records_count,
            Bookmark: metadata.bookmark,
        };

        return results;
    }

    /**
     * 生成返回结果
     */
    async generateResults(iterator, isHistory) {
        let allResults = [];
        while (true) {
            let res = await iterator.next();

            if (res.value && res.value.value.toString()) {
                let jsonRes = {};
                console.info('res数据为：\n' + res.value.value.toString('utf8'));

                if (isHistory && isHistory === true) {
                    jsonRes.TxId = res.value.tx_id;
                    jsonRes.Timestamp = res.value.timestamp;
                    jsonRes.IsDelete = res.value.is_delete.toString();
                    try {
                        jsonRes.Value = JSON.parse(res.value.value.toString('utf8'));
                    } catch (err) {
                        console.info('res历史数据解析异常：\n' + err);
                        jsonRes.Value = res.value.value.toString('utf8');
                    }
                } else {
                    jsonRes.Key = res.value.key;
                    try {
                        jsonRes.Record = JSON.parse(res.value.value.toString('utf8'));
                    } catch (err) {
                        console.info('res数据解析异常：\n' + err);
                        jsonRes.Record = res.value.value.toString('utf8');
                    }
                }
                allResults.push(jsonRes);
            }
            if (res.done) {
                console.info('res数据结束');
                await iterator.close();
                console.info('res返回数据为：\n' + allResults);
                return allResults;
            }
        }
    }
}

module.exports = Cnsp;
